package com.sw.patroleditor.service.impl;

import com.alibaba.fastjson.JSON;
import com.sw.patroleditor.component.robot.ctrl.CtrlInterface;
import com.sw.patroleditor.component.robot.message.Move;
import com.sw.patroleditor.domain.vo.RobotModelVO;
import com.sw.patroleditor.domain.vo.RobotMoveCtrlVO;
import com.sw.patroleditor.domain.vo.RobotSpecialCtrlVO;
import com.sw.patroleditor.domain.vo.RobotToolCtrlVO;
import com.sw.patroleditor.exception.BusinessException;
import com.sw.patroleditor.exception.ErrorCode;
import com.sw.patroleditor.service.RobotCtrlService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * Created by shiwn on 2024/7/2 14:05
 */
@Service
public class RobotCtrlServiceImpl implements RobotCtrlService {
    private static final Logger logger = LoggerFactory.getLogger(RobotCtrlServiceImpl.class);

    /**
     * 机器人id
     */
    @Value("${robot.id}")
    private int robotId = 0;
    /**
     * 机器人ip
     */
    @Value("${robot.ip}")
    private String robotIp = null;
    /**
     * 机器人端口
     */
    @Value("${robot.port}")
    private int robotPort = 0;

    /**
     * @Description: 机器人特殊状态控制
     * @Date: 2021/8/6 11:22
     * @Author: shiwn
     * @Param: [vo]
     * @Return:
     */
    @Override
    public boolean specialCtrl(RobotSpecialCtrlVO vo) {
        //  控制
        try {
            String resp = CtrlInterface.connection(robotIp, robotPort).specialtrl(vo.getType());
            //   判断结果
            return isSuccess(resp);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("机器人特殊状态控制失败，异常：{}", e.getMessage());
        }
        return false;
    }

    /**
     * @Description: 机器人移动控制
     * @Date: 2021/7/8 13:57
     * @Author: shiwn
     * @Param: [vo]
     * @Return:
     */
    @Override
    public boolean robotMoveCtrl(RobotMoveCtrlVO vo) {
        //  控制
        Move move = new Move();
        BeanUtils.copyProperties(vo, move);
        try {
            String resp = CtrlInterface.connection(robotIp, robotPort).robotCtrl(move);
            //   判断结果
            return isSuccess(resp);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("机器人移动控制失败，异常：{}", e.getMessage());
        }
        return false;
    }

    /**
     * @Description: 补光灯、雨刷控制
     * @Date: 2021/7/8 14:04
     * @Author: shiwn
     * @Param: [openStatus]
     * @Return:
     */
    @Override
    public boolean toolCtrl(RobotToolCtrlVO vo) {
        //  控制
        String resp = "";
        try {
            if (vo.getType() == 1) {
                //  雨刷
                resp = CtrlInterface.connection(robotIp, robotPort).openWiper(vo.getOpstatus());
            } else {
                //  补光灯
                resp = CtrlInterface.connection(robotIp, robotPort).openLight(vo.getOpstatus());
            }
            //   判断结果
            return isSuccess(resp);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("机器人补光灯状态控制失败，异常：{}", e.getMessage());
        }
        return false;
    }

    /**
     * @Description: 模式切换
     * @Date: 2021/8/10 16:18
     * @Author: shiwn
     * @Param: [vo]
     * @Return:
     */
    @Override
    public boolean modelChange(RobotModelVO vo) {
        try {
            String resp = CtrlInterface.connection(robotIp, robotPort).modeChange(vo.getModel());
            //   判断结果
            return isSuccess(resp);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("机器人模式切换失败，异常：{}", e.getMessage());
        }
        return false;
    }

    /**
     * @Description: 解析机器人返回数据
     * @Date: 2024/7/2 15:36
     * @Author: shiwn
     * @Param: str：机器人返回数据
     */
    private boolean isSuccess(String str) {
        //  判断是否有返回值
        if (org.apache.commons.lang3.StringUtils.isEmpty(str)) {
            throw new BusinessException(ErrorCode.ROBOT_RETURN_NULL);
        }
        //  判断结果
        Map<String, String> map = JSON.parseObject(str, Map.class);
        Integer code = Integer.parseInt(String.valueOf(map.get("code")));
        if (!code.equals(ErrorCode.SUCCESS.getCode())) {
            String msg = String.valueOf(map.get("msg"));
            if (StringUtils.isEmpty(msg)) {
                msg = ErrorCode.OPERATION_FAILED.getMsg();
            }
            throw new BusinessException(1, msg);
        }
        return true;
    }
}
