package com.sw.patroleditor.util;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import java.util.LinkedList;
import java.util.List;

/**
 * xml转换工具类
 */

@Slf4j
public class XmlUtils {


    /**
     * 将对象转为对应的xml报文：可利用注解自定义节点对象名
     *
     * @param object 对象
     * @param classT 类
     * @return xml
     * @throws JAXBException ex
     */
    public static String objectToXml(Object object, Class<?>... classT) throws JAXBException {
        StringWriter writer = new StringWriter();
        JAXBContext context = JAXBContext.newInstance(classT);
        Marshaller marshaller = context.createMarshaller();
        //设置编码格式
        marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
        marshaller.setProperty(Marshaller.JAXB_FRAGMENT, false);
        marshaller.marshal(object, writer);
        return new String(writer.getBuffer());
    }

    /**
     * 将xml报文转换为对象：可按照自定义节点名转换属性
     *
     * @param clazz  clazz
     * @param xmlStr xml
     * @return object
     */
    public static Object xmlToObject(String xmlStr, Class<?>... clazz) {
        Object xmlObject = null;
        try {
            JAXBContext context = JAXBContext.newInstance(clazz);
            Unmarshaller unmarshaller = context.createUnmarshaller();
            StringReader sr = new StringReader(xmlStr);
            xmlObject = unmarshaller.unmarshal(sr);
        } catch (JAXBException e) {
            log.error("xmlStrToObject error:", e);
        }
        return xmlObject;
    }


    /**
     * 按照对象属性默认设置节点
     *
     * @param bean
     * @return
     * @throws JsonProcessingException
     */
    public static String bean2Xml(Object bean) throws JsonProcessingException {
        if (bean == null) return null;
        XmlMapper xmlMapper = new XmlMapper();
        // 将JavaBean对象转换为XML字符串
        String xml = xmlMapper.writeValueAsString(bean);
        return xml;
    }


    /**
     * 按照节点默认恢复对象属性
     *
     * @param xmlStr
     * @return
     * @throws JDOMException
     * @throws IOException
     */
    public static JSONObject xml2Json(String xmlStr) throws JDOMException, IOException {
        if (StringUtils.isEmpty(xmlStr)) {
            return null;
        }
        xmlStr = xmlStr.replaceAll("\\\n", "");
        byte[] xml = xmlStr.getBytes("UTF-8");
        JSONObject json = new JSONObject();
        InputStream is = new ByteArrayInputStream(xml);
        SAXBuilder sb = new SAXBuilder();
        Document doc = sb.build(is);
        Element root = doc.getRootElement();
        json.put(root.getName(), iterateElement(root));

        return json;
    }

    private static JSONObject iterateElement(Element element) {
        List<Element> node = element.getChildren();
        JSONObject obj = new JSONObject();
        List list = null;
        for (Element child : node) {
            list = new LinkedList();
            String text = child.getTextTrim();
            if (StringUtils.isBlank(text)) {
                if (child.getChildren().size() == 0) {
                    continue;
                }
                if (obj.containsKey(child.getName())) {
                    list = (List) obj.get(child.getName());
                }
                list.add(iterateElement(child)); //遍历child的子节点
                obj.put(child.getName(), list);
            } else {
                if (obj.containsKey(child.getName())) {
                    Object value = obj.get(child.getName());
                    try {
                        list = (List) value;
                    } catch (ClassCastException e) {
                        list.add(value);
                    }
                }
                if (child.getChildren().size() == 0) { //child无子节点时直接设置text
                    obj.put(child.getName(), text);
                } else {
                    list.add(text);
                    obj.put(child.getName(), list);
                }
            }
        }
        return obj;
    }
}
